/*
CSC 130, Spring 2011
Weeek 3 #4

Create a Processing program that you can use to draw a picture with selectable
colors. There are several ways to go about this. One is to draw boxes that you
can click on to select the color you want to draw. Another could use the
keyboard (or both). Make sure to provide instructions (in comments, or in a help
message).

Some more hints: make use of the special Processing variables mouseX, mouseY,
and mousePressed. Start out simple (like with only one color), and add as you go
along.

Optional: also provide a way to select brush size and/or shape. Some more ideas:
create special brush types, like the airbrush tool you see in some paint
programs; create an eraser; provide a way to enter text to be drawn on the
screen.

*/

// Constants

// Width of option selector boxes
final int squareWidth = 20;

// Red selector color and box position
final int rCol = color(255, 0, 0);
final int rX = 5;
final int rY = 5;

// Green selector color and box position
final int gCol = color(0, 255, 0);
final int gX = 5;
final int gY = rY + squareWidth;

// Blue selector color and box position
final int bCol = color(0, 0, 255);
final int bX = 5;
final int bY = gY + squareWidth;

// Line type selector position
final int lineX = 5;
final int lineY = bY + squareWidth;

// Circle type selector position
final int circleX = 5;
final int circleY = lineY + squareWidth;

// Airbrush type selector position
final int airX = 5;
final int airY = circleY + squareWidth;

// 1 pixel weight selector position
final int weight1X = 5;
final int weight1Y = airY + squareWidth;

// 5 pixel weight selector position
final int weight5X = 5;
final int weight5Y = weight1Y + squareWidth;

// 10 pixel weight selector position
final int weight10X = 5;
final int weight10Y = weight5Y + squareWidth;

// Tool type codes
final int CIRCLE = 0, LINE = 1, AIRBRUSH = 2;

int col = rCol;  // Current color
int weight = 5;  // Current weight
int drawType = CIRCLE;  // Current tool type

// Initial setup
void setup() {
    size(800, 600);
    frameRate(100);
    smooth();
    background(255);
}

// Draws selector boxes and updates lines/circles drawn each frame
void draw() {
    // If mouse is pressed, draw a new line/circle/etc., based 
    // on current color, weight, and tool type
    if (mousePressed) {
        fill(col);
        stroke(col);
        if (drawType == CIRCLE) {
            // Draw circle at current location
            noStroke();
            ellipse(mouseX, mouseY, weight, weight);
        } else if (drawType == LINE) {
            // Draw line from last location to current location
            strokeWeight(weight);
            line(pmouseX, pmouseY, mouseX, mouseY);
        } else if (drawType == AIRBRUSH) {
            // Draw a random semi-transparent circle to simulate an airbrush
            noStroke();
            // Random diameter
            float diam = random(weight * 2);
            // Semi-transparent version of current color
            fill(red(col), green(col), blue(col), 50);
            // Draw circle at random small offset from current position
            ellipse(mouseX + random(weight * 2), mouseY + random(weight * 2),
                    diam, diam);
        }
    }

    
    // Draw color boxes
    stroke(0);
    strokeWeight(1);
    fill(rCol);
    rect(rX, rY, squareWidth, squareWidth);
    fill(gCol);
    rect(gX, gY, squareWidth, squareWidth);
    fill(bCol);
    rect(bX, bY, squareWidth, squareWidth);

    // Draw line type boxes
    fill(255);
    rect(lineX, lineY, squareWidth, squareWidth);
    line(lineX, lineY, lineX + squareWidth, lineY + squareWidth);
    rect(circleX, circleY, squareWidth, squareWidth);
    ellipse(circleX + squareWidth/2, circleY + squareWidth/2, squareWidth, squareWidth);
    rect(airX, airY, squareWidth, squareWidth);
    fill(0, 0, 0, 50);
    ellipse(airX + squareWidth/2, airY + squareWidth/2, squareWidth, squareWidth);

    // Draw weight boxes
    fill(255);
    rect(weight1X, weight1Y, squareWidth, squareWidth);
    rect(weight5X, weight5Y, squareWidth, squareWidth);
    rect(weight10X, weight10Y, squareWidth, squareWidth);
    fill(0);
    ellipse(weight1X + squareWidth/2, weight1Y + squareWidth/2, 1, 1);
    ellipse(weight5X + squareWidth/2, weight5Y + squareWidth/2, 5, 5);
    ellipse(weight10X + squareWidth/2, weight10Y + squareWidth/2, 10, 10);
}

// If mouse is clicked, check if current location is within any of the 
// selector boxes, and update drawing option appropriately
void mouseClicked() {
    // Select new color
    if (mouseX >= rX && mouseX <= rX + squareWidth
        && mouseY >= rY && mouseY <= rY + squareWidth) {
        col = rCol;
    } else if (mouseX >= gX && mouseX <= gX + squareWidth
               && mouseY >= gY && mouseY <= gY + squareWidth) {
        col = gCol;
    } else if (mouseX >= bX && mouseX <= bX + squareWidth
               && mouseY >= bY && mouseY <= bY + squareWidth) {
        col = bCol;
    } 
    // Select new tool type
    else if (mouseX >= lineX && mouseX <= lineX + squareWidth
               && mouseY >= lineY && mouseY <= lineY + squareWidth) {
        drawType = LINE;
    } else if (mouseX >= circleX && mouseX <= circleX + squareWidth
               && mouseY >= circleY && mouseY <= circleY + squareWidth) {
        drawType = CIRCLE;
    } else if (mouseX >= airX && mouseX <= airX + squareWidth
               && mouseY >= airY && mouseY <= airY + squareWidth) {
        drawType = AIRBRUSH;
    } 
    // Select new weight
    else if (mouseX >= weight1X && mouseX <= weight1X + squareWidth
               && mouseY >= weight1Y && mouseY <= weight1Y + squareWidth) {
        weight = 1;
    } else if (mouseX >= weight5X && mouseX <= weight5X + squareWidth
               && mouseY >= weight5Y && mouseY <= weight5Y + squareWidth) {
        weight = 5;
    } else if (mouseX >= weight10X && mouseX <= weight10X + squareWidth
               && mouseY >= weight10Y && mouseY <= weight10Y + squareWidth) {
        weight = 10;
    }
}
