/*
CSC 130, Spring 2011
Weeek 5 part 2 #2


Write a Processing program that implements the following game: when starting the
program, a circle is drawn in a random location. When you click on the circle,
it disappears, and a new circle is drawn in another random location. The game
lasts until you click 10 circles, and your final score is the time it took you
to finish.

Optional: display the elapsed time; have the circles move around; have the
circles move around in a way that tries to avoid the mouse pointer; get creative
with the type of target you’re using. The millis() function is useful for
measuring elapsed time.

Version 2: includes some optional parts, like target movement and drawing current
time and progress. 

*/

// Track number of circles clicked
int hits;
// Indicate if game is over
boolean gameOver;
// Location of current circle
float circleX;
float circleY;
// Location that the current circle is moving towards
float targetX, targetY;
// Start and end times, in ms.
float startTime, endTime;

// Circle max speed while moving
final float MAXSPEED = 30;
// Diameter of target circles
final float DIAM = 100;
// Number of circles clicked to win
final int GOAL = 10;
// Define probability of circle moving to a new location each frame
final float MOVECHANCE = 0.05;

color circleCol = color(0);

void setup() {
    println("A " + true);
    size(500, 500);
    smooth();
    fill(0);
    stroke(255);
    hits = 0;
    gameOver = false;
    // Set initial circle position
    circleX = random(DIAM/2, width - DIAM/2);
    circleY = random(DIAM/2, height - DIAM/2);
    targetX = circleX;
    targetY = circleY;
    circleCol = color(random(256), random(256), random(256));
    // Track starting time
    startTime = millis();
}
    
void draw() {
    // Clear screen
    background(255);
    // If game is over, print score and return from function
    if (gameOver) {
        String message = "Your time was " + (endTime - startTime)/1000 + " seconds.";
        textAlign(CENTER, CENTER);
        text(message, width/2, height/2);
        return;
    }

    // Choose random number in [0, 1), and if it below MOVECHANCE, choose a
    // new random target destination for circle.
    if (random(1) < MOVECHANCE) {
        targetX = random(width);
        targetY = random(height);
    }

    // Handle circle movement. The circle always has a current location and a
    // target location, and will move towards the target.

    // Direction to target location
    float dx = targetX - circleX;
    float dy = targetY - circleY;
    // Distance to target location
    float dist = sqrt(dx * dx + dy * dy);
    // Speed -- slow down as circle approaches target.
    float speed = min(MAXSPEED, dist/20.0);
    // Check to make sure don't divide by 0.
    if (dist > 0) {
        // (dx, dy)/dist is the unit vector pointing towards the target
        // location.
        circleX = circleX + speed * dx / dist;
        circleY = circleY + speed * dy / dist;
    }

    // Draw target circle
    fill(circleCol);
    ellipse(circleX, circleY, DIAM, DIAM);

    // Draw elapsed time and hit count
    fill(0);
    textAlign(LEFT, TOP);
    text("Elapsed time: " + (millis() - startTime)/1000, 2, 2);
    textAlign(RIGHT, TOP);
    text("Hits: " + hits + " / " + GOAL, width - 2, 2);
}

void keyPressed() {
    // If r is pressed, call setup() to restart game
    // Note: I'm not sure if it's proper to call setup() a second time this way.
    if (key == 'r') {
        setup();
    }
}

void mouseClicked() {
    // Return from function (don't do anything) if game is over
    if (gameOver) {
        return;
    }
    // Determine if click was inside target by using the equation for a circle
    // centered at (circleX, circleY).
    float dx = mouseX - circleX;
    float dy = mouseY - circleY;
    float r = DIAM/2;
    if (dx * dx + dy * dy <= r * r) {
        // Increment hits and determine if game is over
        hits = hits + 1;
        gameOver = hits == GOAL;
        if (gameOver) {
            // Keep track of ending time
            endTime = millis();
        } else {
            // If game is not over, pick a new location for the target
            circleX = random(DIAM/2, width - DIAM/2);
            circleY = random(DIAM/2, height - DIAM/2);
            targetX = circleX;
            targetY = circleY;
            // Pick random color for circle
            circleCol = color(random(256), random(256), random(256));
        }
    }
}
